/*
 *  Cryptographic "Data-as-Key" Feedback Analyser
 *
 *  Copyright (C) 2005, 2008 Luke Kenneth Casson Leighton
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*#define TEST_HISTOGRAM*/

#define K_SZ 16
#define D_SZ 16

#ifdef TEST_RIJNDAEL
#include "aes.h"
#define ALG "rjd"
#endif

#ifdef TEST_SERPENT
#include "serpent.h"
#define ALG "serpent"
#endif

#ifdef TEST_DES
#include <openssl/des.h>
#define ALG "des"
#undef K_SZ
#undef D_SZ
#define K_SZ 7
#define D_SZ 8

static void
str_to_key(const unsigned char *str, unsigned char *key)
{
	int i;

	key[0] = str[0]>>1;
	key[1] = ((str[0]&0x01)<<6) | (str[1]>>2);
	key[2] = ((str[1]&0x03)<<5) | (str[2]>>3);
	key[3] = ((str[2]&0x07)<<4) | (str[3]>>4);
	key[4] = ((str[3]&0x0F)<<3) | (str[4]>>5);
	key[5] = ((str[4]&0x1F)<<2) | (str[5]>>6);
	key[6] = ((str[5]&0x3F)<<1) | (str[6]>>7);
	key[7] = str[6]&0x7F;
	for (i=0;i<8;i++) {
		key[i] = (key[i]<<1);
	}
}

#endif

#ifdef TEST_BF
#include <openssl/blowfish.h>
#define ALG "blowfish"
#endif

#ifdef TEST_CAST
#include <openssl/cast.h>
#define ALG "cast"
#endif

#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <getopt.h>


#define sqrt2 1.41421356237309504880

/*
 * the basic premise of this program USED TO BE as follows:
 *
 * for a given initial key "k" and initial data "buf",
 * repeat num_tests times:
 *
 * 	o = E(k, buf); k = o; buf = xor(buf, o)
 *
 * if the algorithm is any good, then using the key as the
 * data for the next block will show no signs of non-randomness
 * in the output.  this just seems like good common sense.
 *
 * (and Rijndael/128 shows problems on blocks as quickly as 5-10 loops)
 *
 * signs of non-randomness aren't such a big hairy deal... as long as
 * there's no correlation between bits being set in the initial key
 * and bits being set in the initial data... and that's what the
 * histogram is there to find out.
 *
 * use gnuplot in weird and wonderful ways to look at the plots in 3d...
 *
 */

extern int num_bits(unsigned char *data, int len);
extern double erfc(double x);

void write_data(unsigned char *d, int len)
{
	int i;
	for (i = 0; i < len; i++)
	{
		printf("%c", d[i]);
	}
	printf("\n");
	fflush(stdout);
}

void print_data(unsigned char *d, int len)
{
	int i;
	for (i = 0; i < len; i++)
	{
		printf("%02x ", d[i]);
	}
	printf("\n");
	fflush(stdout);
}

double data_test(unsigned char *test_data, int len)
{
	int count;
	double sobs;
	double testval;
	double f;

	/* count the number of bits, create a p-value, blah blah */
	count = num_bits(test_data, len);
	testval = (count - ((double)len*4)) * 2;
	sobs = fabs(testval) / sqrtl(len*8);
	f = sobs / sqrt2;
	return erfc(f);
}

extern int gen_rand_impl(unsigned char *, unsigned int len);

static int gen_weighted_pseud(unsigned char *d, const float *keyprob)
{
	unsigned int prng[8*K_SZ*4];
	unsigned char *p = (unsigned char*)prng;
	int i;
	int count = 0;

	gen_rand_impl(p, 8*4*K_SZ);

	for (i = 0; i < K_SZ; i++)
		d[i] = 0;
	for (i = 0; i < 8*K_SZ; i++)
	{
		int tst = ((float)prng[i]) < (keyprob[i] * ((float)0xFFFFFFFF));
		if (tst)
		{
			bit_set(d, i);
			count++;
		}
	}
	return 4 * K_SZ;
	return count;
}

void weighted_pseud(unsigned char *d, const float *keyprob)
{
	int count;
	do {
		count = gen_weighted_pseud(d, keyprob);
	} while (count != 4 * K_SZ);

	printf("weighted pseud key: ");
	write_data(d, K_SZ);
	fflush(stdout);
}


void read_input(unsigned char *input, int len)
{
	int i;
	for (i = 0; i < len; i++)
	{
		fscanf(stdin, "%c", &input[i]);
	}
	/*print_data(input, D_SZ);*/
}

